            <h3>Download</h3>
            <p><a href="{{ pathto('download') }}">Download these documents</a></p>
	    <h3>Docs for other versions</h3>
	    <ul>
	      <li><a href="http://docs.python.org/3.3/">Python 3.3 (stable)</a></li>
	      <li><a href="http://docs.python.org/3.4/">Python 3.4 (in development)</a></li>
              <li><a href="http://www.python.org/doc/versions/">Old versions</a></li>
            </ul>

            <h3>Other resources</h3>
            <ul>
              {# XXX: many of these should probably be merged in the main docs #}
              <li><a href="http://www.python.org/dev/peps/">PEP Index</a></li>
              <li><a href="http://wiki.python.org/moin/BeginnersGuide">Beginner's Guide</a></li>
              <li><a href="http://wiki.python.org/moin/PythonBooks">Book List</a></li>
              <li><a href="http://www.python.org/doc/av/">Audio/Visual Talks</a></li>
            </ul>
